# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Any, Dict, List, Optional, Tuple, Union

from django.conf import settings
from django.db import models
from rest_framework.serializers import ModelSerializer


class BaseRoleAdapter(ABC):
    """Abstract base class for role adapters.

    This class defines the interface for role adapters used in the `ApiRole` class. Subclasses of this class should
    implement the methods defined here to customize role-based permissions and behavior for specific views and actions.
    """

    check_role_permissions: bool = True
    serializer_class: Optional[ModelSerializer] = None
    status_field: str = "status"
    status_flow: Dict[Any, List[Any]] = settings.THX_ROLE_DEFAULT_STATUS_FLOW

    def __init__(self, ar, *args, **kwargs):
        """BaseRoleAdapter constructor.

        :param ar: the `ApiRole` instance
        :param args: additional positional arguments
        :param kwargs: additional keyword arguments
        """
        self.ar = ar
        super().__init__(*args, **kwargs)

    def get_queryset(self):
        """Get the queryset for the role adapter.

        TODO: implement this method if managing the `get_queryset` of `ThuxModelMixin` using filter backends.

        :return: the queryset or NotImplemented
        """
        return NotImplemented

    def get_serializer_class(self) -> Optional[ModelSerializer]:
        """Get the serializer class for the role adapter.

        Override this method only if you need to customize the view serializer class.

        :return: the serializer class or None
        """
        return self.serializer_class

    def get_response_serializer_class(self, request_serializer: ModelSerializer) -> Optional[ModelSerializer]:
        """Get the response serializer class for the role adapter.

        Override this method only if you need to customize the response serializer class.

        :param request_serializer: the current serializer
        :return: the response serializer class or None
        """

    def get_from_status(self, instance):
        """Get the "from status" of the role adapter for the given instance.

        :param instance: the model instance
        :return: the "from status" of the instance or None
        """
        return getattr(instance, self.status_field, None)

    def get_to_status(self, from_status) -> List[int]:
        """Get the "to status" of the role adapter for the given "from status".

        :param from_status: the current status
        :return: list of "to status" or an empty list
        """
        return self.status_flow.get(from_status, [])

    @abstractmethod
    def filter_queryset(self, request, queryset, view) -> Tuple[models.QuerySet, bool]:
        """Filter the queryset based on the user's permissions.

        Subclasses should implement this method to customize queryset filtering based on user roles.

        :param request: the restframework request
        :param queryset: the original queryset
        :param view: the restframework view
        :return: a tuple containing the filtered queryset and a boolean indicating whether filtering was applied.
        """
        return queryset, False

    @abstractmethod
    def has_permission(self, request, view) -> Tuple[bool, bool]:
        """ Check if the user has permission to perform the action.

        Subclasses should implement this method to define the logic for checking permissions.

        :param request: the restframework request
        :param view: the restframework view
        :return: a tuple containing a boolean indicating whether the user has permission and a boolean indicating
        whether permission was checked
        """

    @abstractmethod
    def get_required_fields(self) -> List[str]:
        """Get the required fields for the user for the app, model, profile, from_status, or request assigned.

        Subclasses should implement this method to provide the required fields based on user roles.

        :return: list of required fields
        """

    @abstractmethod
    def get_used_fields(self) -> Union[List[str], str]:
        """Get the used fields for the user for the app, model, profile, from_status, or request assigned.

        Subclasses should implement this method to provide the used fields based on user roles.

        :return: list of used fields or "__all__" for all fields
        """


class UsableAdapterMixin:
    """Mixin class for Usable Adapters.

    This mixin class provides default implementations for the methods defined in the `BaseRoleAdapter` class.
    """

    def filter_queryset(self, request, queryset, view) -> Tuple[models.QuerySet, bool]:
        """Default implementation of filtering the queryset.

        This method returns the original queryset and indicates that filtering was not applied.

        :param request: the restframework request
        :param queryset: the original queryset
        :param view: the restframework view
        :return: a tuple: (original queryset, queryset_filtered (False))
        """
        return queryset, False

    def has_permission(self, request, view) -> Tuple[bool, bool]:
        """Default implementation of checking permission.

        This method always returns True and indicates that permission was not checked.

        :param request: the restframework request
        :param view: the restframework view
        :return: a tuple of boolean values: (has_permission, is_permission_checked)
        """
        return True, False

    def get_required_fields(self):
        """Default implementation of getting required fields.

        This method returns an empty list of required fields.

        :return: empty list
        """
        return []

    def get_used_fields(self):
        """Default implementation of getting used fields.

        This method returns "__all__" indicating all fields are used.

        :return: "__all__"
        """
        return "__all__"


# READY TO USE ADAPTERS
class BaseAdapter(UsableAdapterMixin, BaseRoleAdapter):
    """Base adapter class for role-based permissions.

    This adapter class combines the `UsableAdapterMixin` and the `BaseRoleAdapter` to provide a usable base class for
    custom role adapters. It is ready to use for basic role-based permission checks.
    """


class NoStatusFlowAdapter(BaseAdapter):
    """Adapter class for role-based permissions with no status flow.

    This adapter class inherits from `BaseAdapter` and overrides the `get_from_status` method to return None for the
    "from status", effectively disabling the status flow for role-based permissions.
    """

    def get_from_status(self, instance):
        """Override the `get_from_status` method to return None.

        This effectively disables the status flow for role-based permissions.

        :param instance: the model instance
        :return: None
        """
        return None


class NoneAdapter(BaseAdapter):
    """Adapter class for role-based permissions with no permission and no queryset.

    This adapter class inherits from `BaseAdapter` and overrides the `filter_queryset` and `has_permission` methods to
    return an empty queryset and False, respectively. This effectively denies all permissions for any action.
    """

    def filter_queryset(self, request, queryset, view) -> Tuple[models.QuerySet, bool]:
        """Override the `filter_queryset` method to return an empty queryset and True.

        This effectively denies all permissions for any action.

        :param request: the restframework request
        :param queryset: the original queryset
        :param view: the restframework view
        :return: a tuple (none queryset, queryset_filtered)
        """
        return queryset.none(), True

    def has_permission(self, request, view) -> Tuple[bool, bool]:
        """Override the `has_permission` method to return False and True.

        This effectively denies all permissions for any action.

        :param request: the restframework request
        :param view: the restframework view
        :return: a tuple of boolean values: (has_permission, is_permission_checked)
        """
        return False, True
