# -*- coding: utf-8 -*-
from rest_framework.filters import BaseFilterBackend


class ThxRoleFilterBackend(BaseFilterBackend):
    """Filter backend for role-based permissions in the thx-role app.

    This filter backend is responsible for filtering the queryset based on role-based permissions defined in the
    `ApiRole` class. It delegates the filtering to the `filter_queryset` method of the associated `ApiRole` instance.
    If the queryset is not filtered by the `ApiRole`, it falls back to the `default_filter_queryset` method, which
    raises a NotImplementedError. Subclasses of `ThxRoleFilterBackend` should provide a custom implementation for
    `default_filter_queryset` to handle the default behavior.
    """

    def filter_queryset(self, request, queryset, view):
        """Filter the queryset based on role-based permissions.

        This method delegates the filtering to the `filter_queryset` method of the associated `ApiRole` instance.
        If the queryset is not filtered by the `ApiRole`, it falls back to the `default_filter_queryset` method.

        :param request: the restframework request
        :param queryset: the original queryset
        :param view: the restframework view
        :return: the filtered queryset
        """
        queryset, filtered = view.ar.filter_queryset(request, queryset, view)
        if not filtered:
            return self.default_filter_queryset(request, queryset, view)
        return queryset

    def default_filter_queryset(self, request, queryset, view):
        """Default method for filtering the queryset.

        Subclasses should implement this method to provide custom filtering behavior when the `ApiRole` does not
        filter the queryset.

        :param request: the restframework request
        :param queryset: the original queryset
        :param view: the restframework view
        :raise: NotImplementedError
        """
        raise NotImplementedError()
